#        name: dotted decimal
# description: generate dotted decimal notation representation of a non-negative integral value
#      author: Kai Burghardt <hackerman​@﻿Kai﻿Burghardt​.﻿de>
#   copyright: public domain, fall back to Creative Commons license zero

function standard::dotted_decimal(
	Integer[0] $value, Variant[Integer[2], Default] $base = default, Integer[1] $minimum_number_of_groups = 1
) >> String {
	if $base == default {
		# generate a list of zeros
		$zero = Integer[1, $minimum_number_of_groups].step(1).map |$i| { 0 }
		
		# concatenate the list of zeros and add $value to the last element
		$group = $zero[1, $minimum_number_of_groups - 1] + [$value]
		
		# join them for dotted decimal notation, this also stringifies the integers
		$result = join($group, '.')
	} else {
		# how many groups are required at least?
		$required_groups = ceiling(inline_template("<%= Math.log(${value}, ${base}) %>"))
		# vs. the number of groups desired:
		$number_of_groups = max($required_groups, $minimum_number_of_groups)
		
		# repeatedly do Horner’s method
		$group = Integer[1, $number_of_groups].step(1).map |$group_number| {
			Integer[1, $group_number].step(1).reduce($value) |$memo, $iteration| {
				if $iteration == $group_number {
					$memo % $base
				} else {
					floor($memo / $base)
				}
			}
		}
		
		# read numbers in reverse and join them with a `.`
		$result = $group.reverse_each.convert_to(Array).join('.')
	}
	
	# return result
	$result
}
# EOF
