#        name: internet protocol version 4 address
# description: obtains the numeric value of an IPv4 address from a string representation
#      author: Kai Burghardt <hackerman​@﻿Kai﻿Burghardt​.﻿de>
#   copyright: public domain, fall back to Creative Commons license zero

function standard::internet_protocol_version_4_address (
	Pattern['\A(?i:[0-9A-FX.])+\z'] $n
) >> Standard::InternetProtocolVersion4address {
	$part = $n.split(/\./)
	
	case $part {
		Array[Any, 0, 0]: {
			fail("standard::internet_protocol_version_4_address('${n}') failed: string '${n}' is empty")
		}
		Array[String, 1, 1]: {
			$result                               = Integer[0, 0xFFFFFFFF].new($part[0])
		}
		Array[String, 2, 2]: {
			$class_a_network                      = Integer[0, 0xFF].new($part[0])
			$class_a_host                         = Integer[0, 0xFFFFFF].new($part[1])
			
			$result                               = $class_a_network * 0x1000000 + $class_a_host
		}
		Array[String, 3, 3]: {
			$first_part                           = Integer[0, 0xFF].new($part[0])
			$second_part                          = Integer[0, 0xFF].new($part[1])
			$host                                 = Integer[0, 0xFFFF].new($part[2])
			
			$result                               = $first_part * 0x1000000 + $second_part * 0x10000 + $host
		}
		Array[String, 4, 4]: {
			$octet_0                              = Integer[0, 0xFF].new($part[0])
			$octet_1                              = Integer[0, 0xFF].new($part[1])
			$octet_2                              = Integer[0, 0xFF].new($part[2])
			$octet_3                              = Integer[0, 0xFF].new($part[3])
			
			$result                               = $octet_0 * 0x1000000 + $octet_1 * 0x10000 + $octet_2 * 0x100 + $octet_3
		}
		default: {
			fail("string '${n}' contains too many dot-separated parts")
		}
	}
	
	$result
}
# EOF
