#        name: fully-qualified domain name
# description: provides an FQDN observing the LDH (letters, digits, hyphen) rule.
#      author: Kai Burghardt <hackerman​@﻿Kai﻿Burghardt​.﻿de>
#   copyright: public domain, fall back to Creative Commons license zero

type Standard::FullyQualifiedDomainName = Pattern[
		# ABC.DEF.GHI.JKL.         up to four labels
		'\A[A-Za-z][A-Za-z0-9]{,61}(\.[A-Za-z][A-Za-z0-9]{,62}){,3}\.\z',
		
		# A-B.C-D.E-F.G-H.         up to four labels containing dashes in the middle
		'\A[A-Za-z][A-Za-z0-9\-]{,60}[A-Za-z0-9](\.[A-Za-z][A-Za-z0-9\-]{,61}[A-Za-z0-9]){,3}\.\z',
		
		# A.B.C.D.E.               up to five labels – as of 2024 the longest internationalized ccTLD is 24 octets long
		'\A[A-Za-z][A-Za-z0-9\-]{,55}[A-Za-z0-9](\.[A-Za-z][A-Za-z0-9\-]{,54}[A-Za-z0-9]){,2}\.[A-Za-z][A-Za-z0-9\-]{,22}[A-Za-z0-9]\.\z',
		
		# 52.0.2.10.IN-ADDR.ARPA.  “Internet address 10.2.0.52” [qoute source: RFC 1035]
		'\A([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5]\.){,4}(?i:in-addr\.arpa)\.\z'
	]
