#        name: symbolic mode
# description: provides a regular expression matching a symbolic file mode suitable for `chmod(1)`
#      author: Kai Burghardt <hackerman​@﻿Kai﻿Burghardt​.﻿de>
#   copyright: public domain, fall back to Creative Commons license zero

# Extended-Backus-Naur-Form for reference.
# 
#     symbolic mode = clause, { ",", clause } ;
#   
#       action list = action, { action } ;
#   permission list = permission, { permission } ;
#   
#            clause = { who }, action list ;
#            action = operation, [ permission list | permission copy ] ;      (* Note: permission list/copy are optional ! *)
#   
#               who = 'u' | 'g' | 'o' | 'a' ;
#         operation = '+' | '-' | '=' ;
#   permission copy = 'u' | 'g' | 'o' ;
#        permission = 'r' | 'w' | 'x' | 'X' | 's' | 't' ;                   (* The 't' (“sticky bit”) is an XSI extension. *)

type Standard::SymbolicFileMode = Pattern['\A([ugoa]*([+=\-]([ugo]|[rwxXst]+)?)+)(,([ugoa]*([+=\-]([ugo]|[rwxXst]+)?)+))*\z']
#                                            ││    │  │    │ │   │ │      │    ││││        │      └ perm list/cp ─┘ │  │
# op    stands for  operation                │└who ┘  └ op ┘ └cp ┘ └ perm ┘    ││││        └── action ──────────────┘  │
# cp    stands for  copy                     │         at least one action ────┘││└─── separating comma                │
# perm  stands for  permission               └──── clause ──────────────────────┘└──── clause (repetition) ────────────┘
